/* parse.y -- Command line parser */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

%{
  #include <stdio.h>
  #include <readline/readline.h>
  #include "shell/pr0nsh.h"
  #include "parse/ast.h"
  #include "util/debug.h"
  #include "util/llist.h"
  #include "util/symbol.h"
  #include "shell/redir.h"

  extern int errno;
  static void my_yydebug(void);
  static ASTCommand *cmd;
%}
%union {
  char *str;
  LinkedList *list;
  ASTCommand *cmd;
  REDIR *redir;
}

%token <str> VAR DOLLAR CR STRING BANG AMP EQ BTICK PIPE
%token <redir> REDIR_TOK
%type  <str>  expr
%type  <list> exprlist
%type  <cmd> func history onecmd redir_cmd pipe_cmd
%type  <redir> redir
%right DOLLAR
%%

cmd: /* nothing */
      | cmd CR                { ASTExecute(NULL); }
      | cmd pipe_cmd CR       { ASTExecute($2); 
                                my_yydebug(); }
      | cmd pipe_cmd AMP CR   { ASTCommand *tmp = $2;
                                while(tmp->next) tmp=tmp->next;
                                tmp->bg = 1;
                                ASTExecute($2);
                                my_yydebug(); }
      | cmd error             { yyerrok; }
      ;

pipe_cmd:   redir_cmd               { $$ = $1;
                                      $1->bg = 0;
                                      $1->quiet = 0;
                                      $1->next = NULL; }
          | pipe_cmd PIPE redir_cmd { ASTCommand *tmp = $1;
                                      while(tmp->next) tmp=tmp->next;
                                      tmp->bg = 1;
                                      tmp->quiet = 1;
                                      tmp->next = $3;
                                      $3->next = NULL;
                                      $3->bg = 0;
                                      $3->quiet = 0;
                                      $$ = $1; }
          ;

redir_cmd:  onecmd       { $1->redir = NULL;
                            $$ = $1; }
          | onecmd redir { $1->redir = $2;
                            $$ = $1; }
          ;

redir:    REDIR_TOK expr        { $$ = $1;
                                  $$->filename = strdup($2); 
                                  $$->fd = -1; }
        | redir REDIR_TOK expr  { $$ = $2;
                                  $$->next = $1;
                                  $$->filename = strdup($3); 
                                  $$->fd = -1; }
        ;

onecmd:   func           { $$ = $1; }
        | history        { $$ = $1; }
        ;

history: BANG expr
           { $$ = newAST();
             $$->type = AST_HIST;
             $$->expr = strdup($2); } 
      ;

func:    exprlist  { $$ = newAST();
                     $$->type = AST_FUNC;
                     $$->cmd = strdup($1->s);
                     $$->args = $1->next; }
       | expr      { $$ = newAST();
                     $$->type = AST_FUNC;
                     $$->cmd = strdup($1);
                     $$->args = NULL; }
       ;

exprlist:   expr expr      { LinkedList *list;
                             list = createll($1);
                             list->next = createll($2); 
                             $$ = list; }
          | expr exprlist  { LinkedList *list;
                             list = createll($1);
                             list->next = $2;
                             $$ = list; }
          ;

expr:   VAR                 { if (!VarExists($1)) $$ = "";
                              else $$ = VarValue($1); }
      | STRING              { $$ = $1; }
      ;
%%

#include <stdio.h>
#include <signal.h>
#include <setjmp.h>

jmp_buf begin;
char *progname;
static int _lineno = 1;
extern FILE *yyin;
int _run_from_script = 0;

main(int argc, char *argv[])
{
  if( argc > 2 ) {
    fprintf(stderr, "Too many arguments specified.  Usage: cs11sh <script>\n");
    exit(0);
  }
  if( argc == 2 ) {
    if(!strcmp(argv[1], "0")) { printf("%d", 0/0); } // For testing
    setInteractive(0);
    if((rl_instream = fopen(argv[1],"rt")) == NULL) {
      fprintf(stderr, "File \"%s\" does not exist\n", argv[1]);
      exit(0);
    }
  }
  else setInteractive(1);
  progname = argv[0];
  init();
  my_yydebug();
  setjmp(begin);
  yyparse();
  if (isInteractive()) printf("\n");
}

yyerror(char *s)
{
  warning(s, (char *) 0);
}

void my_yydebug(void)
{
  char *str;

  #ifdef YYDEBUG
    str = VarValue("YYDEBUG");
    if(str == NULL) {
      fatal("ERROR!!!  YYDEBUG not defined!\n");
    }
    yydebug = atoi(str);
    free(str);
  #endif
}

int getLineNumber(void)
{
  return _lineno;
}

void setLineNumber(int n)
{
  _lineno = n;
}
