/* process.c -- Functions for dealing with processes. */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <stdio.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "pr0nsh.h"
#include "util/llist.h"
#include "util/debug.h"
#include "shell/process.h"
#include "shell/redir.h"
#include "shell/shell.h"

char *JobStatusDesc(int status);

Job *jobs = NULL;
char JOBSTATUSDESC[5][15] =
  {"Running   ",
   "Done      ",
   "Stopped   ",
   "Suspended ",
   "Terminated"};

int AddJob(int pid, char *cmdline, ASTCommand *ast)
{
  Job *tmp;

  tmp = newJob();
  tmp->JobNum = ++_lastJob;
  tmp->pid = pid;
  tmp->cmdline = strdup(cmdline);
  tmp->status = JOB_RUNNING;
  tmp->changed = 0;
  tmp->next = jobs;
  tmp->ast = ast;
  jobs = tmp;

  return tmp->JobNum;
}

int RemoveJob(Job *job)
{
  Job *cur;
  Job *last = NULL;

  cur = getNextJob(NULL);
  if(cur == job && cur->next == NULL) {
    free(job);
    jobs = NULL;
    return 1;
  }
  while(cur != job && cur) {
    last = cur;
    cur = getNextJob(cur);
  }
  if(cur != job) return 0;
  if(last == NULL) jobs = getNextJob(job);
  else last->next = getNextJob(job);
  free(job);
  return 1;
}

void SetJobStatus(Job *job,int status,int sig)
{
  if(job) {
    job->status = status;
    job->sig = sig;
    setJobChanged(job);
  }
}

int getNumJobs(void)
{
  Job *cur = jobs;
  int count=0;

  while(cur) {
    count++;
    cur = cur->next;
  }
  return count;
}

/* If we are currently running a fg process, stop it */
void SIGTSTPHandler(void)
{
  Job *job;
  doutput("Enter SIGTSTPHandler\n");
  _builtin_jobs(NULL);
  if(getCurrentPID()) {
    AddJob(getCurrentPID(),getCurCMD(),getCurrentAST());
    job = getJobStructFromPID(getCurrentPID());
    SetJobStatus(job,JOB_STOPPED,0);
    _builtin_jobs(createll("-n"));
  }
  doutput("Exit SIGTSTPHandler\n");
}

void SIGCHLDHandler(void)
{
  Job *curjob = jobs,*job;
  int ret,status,sig;
  REDIR *redir;

  while(curjob != NULL) {
    ret = waitpid(curjob->pid,&status,WNOHANG);
    job = getJobStructFromPID(curjob->pid);
    if(ret == curjob->pid) {
      doutput("BACKGROUND process %d terminated: ", ret);
      if(WIFEXITED(status)) {
        doutput("normally, error code: %d\n", WEXITSTATUS(status));
        SetJobStatus(job,JOB_DONE,0);
      }
      if(WIFSIGNALED(status)) {
        sig = WTERMSIG(status);
        doutput("by signal %d (%s)\n", sig, getSigName(sig));
        SetJobStatus(job,JOB_TERMINATED,sig);
      }
      doutput("\n");
      // Now we need to close all file descriptors for the job
      if(curjob->ast) {
//        doutput("Closing file descriptors for cmd \"%s\"\n", curjob->ast->cmd);
        redir = curjob->ast->redir;
        while(redir) {
//          doutput("CLOSE %d in process.c (parent)\n", redir->fd);
//          close(redir->fd);
//          doutput("CLOSE %d in process.c\n", redir->fdclose);
//          if(redir->fdclose != 0) close(redir->fdclose);
          redir=redir->next;
        }
      }
    }
    curjob = curjob->next;
  }
}

Job *getNextJob(Job *job)
{
  if(job) return job->next;
  return jobs;
}

int isJobChanged(Job *job)
{
  assert(job);
  return job->changed;
}

char *getJobStatusDesc(Job *job)
{
  assert(job);
  return JOBSTATUSDESC[getJobStatus(job)];
}

char *getJobCmdline(Job *job)
{
  assert(job);
  return job->cmdline;
}

int getJobStatus(Job *job)
{
  assert(job);
  return job->status;
}

int getJobPID(Job *job)
{
  assert(job);
  return job->pid;
}

int getJobNum(Job *job)
{
  assert(job);
  return job->JobNum;
}

ASTCommand *getJobAST(Job *job)
{
  assert(job);
  return job->ast;
}

void setJobChanged(Job *job)
{
  assert(job);
  job->changed = 1;
}

void clearJobChanged(Job *job)
{
  assert(job);
  job->changed = 0;
}

Job *getJobStructFromJobID(int jobid)
{
  Job *job;

  job = getNextJob(NULL);
  while(job) {
    if(getJobNum(job) == jobid) return job;
    job = getNextJob(job);
  }
  return NULL;
}

Job *getJobStructFromPID(int pid)
{
  Job *job;

  job = getNextJob(NULL);
  while(job) {
    if(getJobPID(job) == pid) return job;
    job = getNextJob(job);
  }
  return NULL;
}

Job *newJob(void)
{
  Job *job;

  job = (Job *) emalloc(sizeof(Job));

  job->cmdline = NULL;
  job->ast = NULL;
  job->next = NULL;

  return job;
}
