/* debug.c -- Functions useful for debugging. */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <setjmp.h>
#include <stdio.h>
#include "util/debug.h"
#include "util/symbol.h"
//#include "util/gc.h"
#include "shell/pr0nsh.h"
#include "shell/redir.h"

extern jmp_buf begin;

void *emalloc(size_t size)
{
  void *p;

  p = malloc(size);
  if (p == 0) {
    errormsg("out of memory", (char *) 0);
    exit(1);
  }
  return p;
}

char *mystrdup(const char *s)
{
  char *newstr;
  int debug, size;

  if(s == NULL) return NULL;
  size = sizeof(char) * (strlen(s) + 1);
  newstr = emalloc(size);
  strcpy(newstr, s);

  return newstr;
}

int _is_debug()
{
  char *str;
  int debug;

  str = VarValue("__DEBUG");
  if(str) {
    debug = atoi(str);
    free(str);
    return debug;
  }
  else return 0;
}

int fatal(const char *format, ...)
{
  char *str;
  va_list ap;
  va_start (ap,format);

  str = emalloc(strlen(format) + 8);
  strcpy(str, "FATAL: ");
  strcat(str, format);
  vfprintf(stderr,str,ap);
  free(str);
  va_end(ap);
  exit(-1);
}

int doutput(const char *format, ...)
{
  int ret = 1;
  char *str;
  va_list ap;
  va_start (ap,format);

  if( _is_debug() ) {
    str = emalloc(strlen(format) + 8);
    strcpy(str, "DEBUG: ");
    strcat(str, format);
    ret = vfprintf(stderr,str,ap);
    free(str);
  }
  va_end(ap);
  return ret;
}

void parseerror(char *s, char *t)
{
  warning(s, t);
  longjmp(begin, 0);
}

void errormsg(char *s, char *t)
{
  warning(s, t);
}

void warning(char *s, char *t)
{
  fprintf(stderr, "pr0nsh: %s", s);
  if (t) fprintf(stderr, " %s", t);
  if (!isInteractive()) fprintf(stderr, " near line %d\n", getLineNumber());
  else fprintf(stderr, "\n");
}

size_t mystrlen(const char *s)
{
  if(s == NULL) return 0;
  return strlen(s);
}

int outputerr(const char *format, ...)
{
  /* Guess we need no more than 100 bytes. */
  int n,size = 100;
  char *p;
  va_list ap;

  if ((p = malloc (size)) == NULL) return -1;
  while (1) {
    /* Try to print in the allocated space. */
    va_start(ap, format);
    n = vsnprintf (p, size, format, ap);
    va_end(ap);
    if((n > -1) && (n < size)) {
      return write(getSTDERRfd(),p,strlen(p));
    }
    if(n > -1) { size = n+1; }
    else { size *= 2; }
    if ((p = realloc (p, size)) == NULL) return -1;
  }
}

int outputout(const char *format, ...)
{
  /* Guess we need no more than 100 bytes. */
  int n,size = 100;
  char *p;
  va_list ap;

  if ((p = malloc (size)) == NULL) return -1;
  while (1) {
    /* Try to print in the allocated space. */
    va_start(ap, format);
    n = vsnprintf (p, size, format, ap);
    va_end(ap);
    if((n > -1) && (n < size)) {
      return write(getSTDOUTfd(),p,strlen(p));
    }
    if(n > -1) { size = n+1; }
    else { size *= 2; }
    if ((p = realloc (p, size)) == NULL) return -1;
  }
}

int output(int fd, const char *format, ...)
{
  /* Guess we need no more than 100 bytes. */
  int n,size = 100;
  char *p;
  va_list ap;

  if ((p = malloc (size)) == NULL) return -1;
  while (1) {
    /* Try to print in the allocated space. */
    va_start(ap, format);
    n = vsnprintf (p, size, format, ap);
    va_end(ap);
    if((n > -1) && (n < size)) {
      return write(fd,p,strlen(p));
    }
    if(n > -1) { size = n+1; }
    else { size *= 2; }
    if ((p = realloc (p, size)) == NULL) return -1;
  }
}

