/* symbol.c -- Functions for dealing with symbol tables */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <stdio.h>
#include <stdlib.h>
#include "util/debug.h"
#include "util/symbol.h"
#include "util/llist.h"
#include "shell/redir.h"
#include "shell/shell.h"

Symbol *symlist;
void _remove_all_vars(Symbol *sp);

Symbol *LookupBuiltin(char *str)
{
  Symbol *sp;

  for (sp = symlist; sp != (Symbol *) 0; sp = sp->next)
    if (strcmp(sp->name, str) == 0 && sp->type == _BLTIN) return sp;
  return NULL;
}

Symbol *getNextSymbol(Symbol *sym)
{
  if(sym) return sym->next;
  return symlist;
}

int isBuiltin(Symbol *sp)
{
  if(sp) {
    if(sp->type == _BLTIN) return 1;
  }
  return 0;
}

int BuiltinExists(char *builtin)
{
  Symbol *sp;

  for (sp = symlist; sp != (Symbol *) 0; sp = sp->next)
    if (strcmp(sp->name, builtin) == 0 && sp->type == _BLTIN) return 1;
  return 0;
}

int VarExists(char *var)
{
  Symbol *sp;

  for (sp = symlist; sp != (Symbol *) 0; sp = sp->next)
    if (strcmp(sp->name, var) == 0 && sp->type == _VAR) return 1;
  return 0;
}

char *getSymbolName(Symbol *sp)
{
  if(sp == NULL) return NULL;
  return (char *)mystrdup(sp->name);
}

char *getSymbolValue(Symbol *sp)
{
  if(sp == NULL) return NULL;
  if(sp->type != _VAR) return NULL;
  return (char *)mystrdup(sp->val);
}

char *VarValue(char *var)
{
  Symbol *sp;
  sp = lookup(var);
  if(sp == NULL) return NULL;
  if(sp->type != _VAR) return NULL;
  return (char *)mystrdup(sp->val);
}

Symbol *AssignVar(char *var, char *val)
{
  Symbol *sp;

  sp = lookup(var);
  if(sp != NULL) {
    if(sp->type != _VAR) {
      outputerr("Error!  Attempting to use reserved keyword \"%s\" as var\n", 
              var);
      return NULL;
    }
    else {
      free(sp->val);
      if(val != NULL) {
        sp->val = (char *)mystrdup(val);
      }
      else { sp->val = NULL; }
      setenv(sp->name,sp->val,1);
    }
  }
  else {
    sp = install(var, _VAR, val);
    setenv(sp->name,sp->val,1);
  }
  if(!strcmp(var,"HOME")) AssignVar("PAD", val);
  return sp;
}

Symbol *lookup(char *s)
{
  Symbol *sp;

  if(symlist == NULL) return NULL;
  for (sp = symlist; sp != (Symbol *) 0; sp = sp->next) {
    if (strcmp(sp->name, s) == 0) return sp;
  }
  return 0;
}

Symbol *install(char *s, int t, char *v)
{
  Symbol *sp;

  sp = newSymbol();
  sp->name = emalloc(strlen(s)+1);
  strcpy(sp->name, s);
  sp->type = t;
  if(v != NULL) sp->val = (char *)mystrdup(v);
  else sp->val = NULL;
  sp->ptr = NULL;
  sp->next = symlist;
  symlist = sp;
  return sp;
}

void RemoveAllVars(void)
{
  _remove_all_vars(symlist);
}

void _remove_all_vars(Symbol *sp)
{
  if(sp == NULL) return;
  _remove_all_vars(sp->next);
  free(sp->name);
  free(sp->val);
  free(sp);
}

int isVar(Symbol *sp)
{
  if(sp)
    if(sp->type == _VAR) return 1;
  return 0;
}

Symbol *newSymbol(void)
{
  Symbol *sym;

  sym = (Symbol *) emalloc(sizeof(Symbol));

  sym->name = NULL;
  sym->val = NULL;
  sym->ptr = NULL;
  sym->next = NULL;

  return sym;
}
